#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/wait.h>
#include <limits.h>
#include <sys/os32.h>
#include <fcntl.h>
#include <string.h>
//----------------------------------------------------------------------
#define RESPAWN_MAX        7
#define COMMAND_MAX      100
#define ARGUMENTS_MAX     32
#define LINE_MAX        1024
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  //
  // `init.c' has its own `init.crt0.s' with a very small stack
  // size. Remember to verify to have enough room for the stack.
  //
  pid_t           pid;
  int             status;
  char           *exec_argv[ARGUMENTS_MAX];
  int             count;
  char           *exec_envp[3];
  char            buffer[LINE_MAX];
  int             r;		// Respawn table index.
  int             b;		// Buffer index.
  size_t          size_read;
  char           *inittab_id;
  char           *inittab_runlevels;
  char           *inittab_action;
  char           *inittab_process;
  int             eof;
  int             fd;
  //
  // It follows a table for commands to be respawn.
  //
  struct
  {
    pid_t           pid;
    char            command[COMMAND_MAX];
  } respawn[RESPAWN_MAX];

  //------------------------------------------------------------------
  signal (SIGHUP, SIG_IGN);
  signal (SIGINT, SIG_IGN);
  signal (SIGQUIT, SIG_IGN);
  signal (SIGILL, SIG_IGN);
  signal (SIGABRT, SIG_IGN);
  signal (SIGFPE, SIG_IGN);
//  signal (SIGKILL, SIG_IGN);  Cannot ignore SIGKILL.
  signal (SIGSEGV, SIG_IGN);
  signal (SIGPIPE, SIG_IGN);
  signal (SIGALRM, SIG_IGN);
  signal (SIGTERM, SIG_IGN);
//  signal (SIGSTOP, SIG_IGN);  Cannot ignore SIGSTOP.
  signal (SIGTSTP, SIG_IGN);
  signal (SIGCONT, SIG_IGN);
  signal (SIGTTIN, SIG_IGN);
  signal (SIGTTOU, SIG_IGN);
  signal (SIGUSR1, SIG_IGN);
  signal (SIGUSR2, SIG_IGN);
  //------------------------------------------------------------------
  printf ("init\n");
  //  heap_clear ();
  //  process_info ();
  //------------------------------------------------------------------
  //
  // Reset the `respawn' table.
  //
  for (r = 0; r < RESPAWN_MAX; r++)
    {
      respawn[r].pid = 0;
      respawn[r].command[0] = 0;
      respawn[r].command[COMMAND_MAX - 1] = 0;
    }
  //
  // Read the `/etc/inittab' file.
  //
  fd = open ("/etc/inittab", O_RDONLY);
  //
  if (fd < 0)
    {
      perror ("Cannot open file `/etc/inittab'");
      exit (-1);
    }
  //
  //
  //
  for (eof = 0, r = 0; !eof && r < RESPAWN_MAX; r++)
    {
      for (b = 0; b < LINE_MAX; b++)
	{
	  size_read = read (fd, &buffer[b], (size_t) 1);
	  if (size_read <= 0)
	    {
	      buffer[b] = 0;
	      eof = 1;		// Close the read loop.
	      break;
	    }
	  if (buffer[b] == '\n')
	    {
	      buffer[b] = 0;
	      break;
	    }
	}
      //
      // Remove comments: just replace `#' with `\0'.
      //
      for (b = 0; b < LINE_MAX; b++)
	{
	  if (buffer[b] == '#')
	    {
	      buffer[b] = 0;
	      break;
	    }
	}
      //
      // If the buffer is an empty string, just loop to next
      // record.
      //
      if (strlen (buffer) == 0)
	{
	  r--;
	  continue;
	}
      //
      //
      //
      inittab_id = strtok (buffer, ":");
      inittab_runlevels = strtok (NULL, ":");
      inittab_action = strtok (NULL, ":");
      inittab_process = strtok (NULL, ":");
      //
      // Only action `respawn' is used.
      //
      if (strcmp (inittab_action, "respawn") == 0)
	{
	  strncpy (respawn[r].command, inittab_process, COMMAND_MAX);
	}
      else
	{
	  r--;
	}
    }
  //
  //
  //
  close (fd);
  //
  // Define common environment.
  //
  exec_envp[0] = "PATH=/bin:/usr/bin:/sbin:/usr/sbin";
  exec_envp[1] = "CONSOLE=/dev/console";
  exec_envp[2] = NULL;
  //
  // Start processes.
  //
  for (r = 0; r < RESPAWN_MAX; r++)
    {
      if (strlen (respawn[r].command) > 0)
	{
	  respawn[r].pid = fork ();
	  if (respawn[r].pid == 0)
	    {
    	      exec_argv[0] = strtok (respawn[r].command, " \t");
              for (count = 1; count < (ARGUMENTS_MAX-2); count++)
                {
    	          exec_argv[count] = strtok (NULL, " \t");
                  if (exec_argv[count] == NULL)
                    {
                      break;
                    }
                }
              //
              // Last element must be NULL, even if there are more
              // arguments than allowed.
              //
              exec_argv[count] = NULL;
              //
              // Run!
              //
	      execve (exec_argv[0], exec_argv, exec_envp);
	      perror (NULL);
	      exit (0);
	    }
	}
    }
  //
  // Wait for the death of child.
  //
  while (1)
    {
      pid = wait (&status);
      for (r = 0; r < RESPAWN_MAX; r++)
	{
	  if (pid == respawn[r].pid)
	    {
	      //
	      // Run it again.
	      //
	      respawn[r].pid = fork ();
	      if (respawn[r].pid == 0)
		{
                  exec_argv[0] = strtok (respawn[r].command, " \t");
                  for (count = 1; count < (ARGUMENTS_MAX-2); count++)
                    {
                      exec_argv[count] = strtok (NULL, " \t");
                      if (exec_argv[count] == NULL)
                        {
                          break;
                        }
                    }
                  //
                  // Last element must be NULL, even if there are more
                  // arguments than allowed.
                  //
                  exec_argv[count] = NULL;
                  //
                  // Run!
                  //
		  execve (exec_argv[0], exec_argv, exec_envp);
		  exit (0);
                }
	      break;
	    }
	}
    }
}
