#include <sys/os32.h>
#include <kernel/proc.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
//----------------------------------------------------------------------
static void usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  pid_t           pid;
  proc_t         *ps;
  int             fd;
  char            stat;
  ssize_t         size_read;
  char            buffer[sizeof (proc_t)];
  unsigned int    min;
  unsigned int    sec;
  size_t          stack_size;
  addr_t          stack_bottom;
  int             stack_usage;
  //
  int             opt;
//  extern char   *optarg;              // not used.
//  extern int      optind;
  extern int      optopt;
  int             option_u = 0;
  int             option_g = 0;
  //
  // Check for options.
  //
  while ((opt = getopt (argc, argv, ":ug")) != -1)
    {
      switch (opt)
	{
	case 'u':
	  option_u = 1;
	  break;
	case 'g':
	  option_g = 1;
	  break;
	case '?':
	  fprintf (stderr, "Unknown option -%c.\n", optopt);
	  usage ();
	  return (1);
	  break;
	case ':':
	  fprintf (stderr, "Missing argument for option -%c\n", optopt);
	  usage ();
	  return (1);
	  break;
	default:
	  fprintf (stderr, "Getopt problem: unknown option %c\n", opt);
	  return (1);
	}
    }
  //
  // Fix options `-u' or `-g'.
  //
  if (option_u)
    option_g = 0;
  if (!option_g)
    option_u = 1;
  //
  // Open `/dev/kmem_ps', to get the process table.
  //
  fd = open ("/dev/kmem_ps", O_RDONLY);
  if (fd < 0)
    {
      printf ("[%s] Cannot open \"/dev/kmem_ps\" ", argv[0]);
      perror (NULL);
      exit (0);
    }
  //
  // Print head.
  //
  if (option_u)
    {
      printf ("pp  p pg                             "
    	      "T * 0x1000 D * 0x1000 stack        \n"
	      "id id rp  tty  uid euid suid usage s "
	      "addr  size addr  size usage    name\n");
    }
  else
    {
      printf ("pp  p pg                             "
    	      "T * 0x1000 D * 0x1000 stack        \n"
	      "id id rp  tty  gid egid sgid usage s "
	      "addr  size addr  size usage    name\n");
    }
  //
  // Scan processes and then print body.
  //
  for (pid = 0; pid < PROCESS_MAX; pid++)
    {
      lseek (fd, (off_t) pid, SEEK_SET);
      size_read = read (fd, buffer, sizeof (proc_t));
      if (size_read < sizeof (proc_t))
	{
	  printf ("[%s] Cannot read \"/dev/kmem_ps\" pid %i ",
		  argv[0], pid);
	  perror (NULL);
	  continue;
	}
      ps = (proc_t *) buffer;
      if (ps->status > 0)
	{
	  ps->name[PATH_MAX - 1] = 0;	// Terminated string.
	  //
	  // Check the current stack size.
	  //
	  if (ps->domain_data == 0)
	    {
	      stack_bottom = ps->domain_text;
	    }
	  else
	    {
	      stack_bottom = ps->domain_data;
	    }
	  //
	  stack_size = stack_bottom - ps->sp;
	  //
	  stack_usage = 100 * stack_size / ps->domain_stack;
	  //
	  switch (ps->status)
	    {
	    case PROC_EMPTY:
	      stat = '-';
	      break;
	    case PROC_CREATED:
	      stat = 'c';
	      break;
	    case PROC_READY:
	      stat = 'r';
	      break;
	    case PROC_RUNNING:
	      stat = 'R';
	      break;
	    case PROC_SLEEPING:
	      stat = 's';
	      break;
	    case PROC_ZOMBIE:
	      stat = 'z';
	      break;
	    default:
	      stat = '?';
	      break;
	    }
	  //
	  min = ((ps->usage / CLOCKS_PER_SEC) / 60);
	  sec = ((ps->usage / CLOCKS_PER_SEC) % 60);
	  //
	  // Print the line.
	  //
	  //
	  // Addresses and sizes are multiple of 4096 (0x1000);
	  // for the stack pointer is shown only the last five
	  // hexadecimal digits.
	  //
	  if (ps->domain_data > 0)
	    {
	      if (option_u)
	        {
	          printf ("%2i %2i %2i %04x %4i %4i %4i %02i.%02i %c "
		          "%05x %04x %05x %04x % 3i%% %15s\n",
		          (unsigned int) ps->ppid,
		          (unsigned int) pid,
		          (unsigned int) ps->pgrp,
		          (unsigned int) ps->device_tty,
		          (unsigned int) ps->uid,
		          (unsigned int) ps->euid,
		          (unsigned int) ps->suid,
		          min, sec, stat,
		          (unsigned int) ps->address_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->domain_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->address_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) (ps->domain_data
		    		          + ps->extra_data)
		          / MEM_BLOCK_SIZE,
		          (unsigned int) stack_usage, ps->name);
	        }
              else
	        {
	          printf ("%2i %2i %2i %04x %4i %4i %4i %02i.%02i %c "
		          "%05x %04x %05x %04x % 3i%% %15s\n",
		          (unsigned int) ps->ppid,
		          (unsigned int) pid,
		          (unsigned int) ps->pgrp,
		          (unsigned int) ps->device_tty,
		          (unsigned int) ps->gid,
		          (unsigned int) ps->egid,
		          (unsigned int) ps->sgid,
		          min, sec, stat,
		          (unsigned int) ps->address_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->domain_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->address_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) (ps->domain_data
		    		          + ps->extra_data)
		          / MEM_BLOCK_SIZE,
		          (unsigned int) stack_usage, ps->name);
	        }
	    }
	  else
	    {
	      if (option_u)
	        {
	          printf ("%2i %2i %2i %04x %4i %4i %4i %02i.%02i %c "
		          "%05x %04x %05x %04x % 3i%% %15s\n",
		          (unsigned int) ps->ppid,
		          (unsigned int) pid,
		          (unsigned int) ps->pgrp,
		          (unsigned int) ps->device_tty,
		          (unsigned int) ps->uid,
		          (unsigned int) ps->euid,
		          (unsigned int) ps->suid,
		          min, sec, stat,
		          (unsigned int) ps->address_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) (ps->domain_text
				          + ps->extra_data)
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->address_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->domain_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) stack_usage, ps->name);
	        }
              else
	        {
	          printf ("%2i %2i %2i %04x %4i %4i %4i %02i.%02i %c "
		          "%05x %04x %05x %04x % 3i%% %15s\n",
		          (unsigned int) ps->ppid,
		          (unsigned int) pid,
		          (unsigned int) ps->pgrp,
		          (unsigned int) ps->device_tty,
		          (unsigned int) ps->gid,
		          (unsigned int) ps->egid,
		          (unsigned int) ps->sgid,
		          min, sec, stat,
		          (unsigned int) ps->address_text
		          / MEM_BLOCK_SIZE,
		          (unsigned int) (ps->domain_text
				          + ps->extra_data)
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->address_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) ps->domain_data
		          / MEM_BLOCK_SIZE,
		          (unsigned int) stack_usage, ps->name);
	        }
	    }
	}
    }
  close (fd);
  return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: ps [-u|g]\n");
}
