#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <stdint.h>
#include <stdbool.h>
//----------------------------------------------------------------------
static void usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  int                status;
  int                sfdn;
  struct sockaddr_in sa;
  ssize_t            read_size;
  ssize_t            sent_size;
  char               buffer[BUFSIZ];
  char              *addr = NULL;
  char              *port = NULL;
  //
  // Arguments.
  //
  if (argc == 3)
    {
      //
      // There are exactly two arguments: destination address and port.
      //
      addr = argv[1];
      port = argv[2];
    }
  else
    {
      //
      // Arguments wrong!
      //
      usage ();
      return (4);
    }
  //
  // Define the destination `sa'
  //
  sa.sin_family = AF_INET;
  sa.sin_port   = htons (atoi (port));
  inet_pton (AF_INET, addr, &sa.sin_addr.s_addr);
  //
  //
  // Open the socket.
  //
  sfdn = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (sfdn < 0)
    {
      perror (NULL);
      return (5);
    }
  //
  // Connect the `sa' destination
  //
  status = connect (sfdn, (struct sockaddr *) &sa, sizeof (sa));
  if (status < 0)
    {
      perror (NULL);
      close (sfdn);
      return (7);
    }
  //
  // Will read from the standard input and send to the other
  // side.
  //
  while (1)
    {
      read_size = read (STDIN_FILENO, buffer, BUFSIZ);
      if (read_size < 0)
        {
          perror (NULL);
          close (sfdn);
          return (8);
        }
      if (read_size == 0)
        {
          close (sfdn);
          return (0);
        }
      //
      sent_size = send (sfdn, &buffer, (size_t) read_size, 0);
      if (sent_size < 0)
        {
          perror (NULL);
          close (sfdn);
          return (9);
        }
      printf ("sent %i bytes\n", (int) sent_size);
    }
  //
  // All done.
  //
  return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: tx_udp DEST_ADDR DEST_PORT\n");
}
