#include <kernel/fs.h>
#include <sys/stat.h>
#include <errno.h>
#include <kernel/lib_k.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
inode_t *
inode_pipe_make (void)
{
    inode_t     *inode;
    //
    // Find a free inode.
    //
    inode = inode_reference ((dev_t) -1, (ino_t) -1);
    if (inode == NULL)
      {
        //
        // No free slot available.
        //
        errset (ENFILE);        // Too many files open in system.
        return (NULL);
      }
    //
    // Put data inside the inode. Please note that `inode->ino' must be
    // zero, because it is necessary to recognize it as an internal
    // inode with no file system. Otherwise, with a value different than
    // zero, `inode_put()' will try to remove it. [*]
    //
    inode->mode           = S_IFIFO;
    inode->uid            = 0;
    inode->gid            = 0;
    inode->size           = 0;
    inode->time           = 0;
    inode->links          = 0;
    inode->direct[0]      = 0;
    inode->direct[1]      = 0;
    inode->direct[2]      = 0;
    inode->direct[3]      = 0;
    inode->direct[4]      = 0;
    inode->direct[5]      = 0;
    inode->direct[6]      = 0;
    inode->indirect1      = 0;
    inode->indirect2      = 0;
    inode->sb_attached    = NULL;
    inode->sb             = 0;
    inode->ino            = 0;   // Must be zero. [*]
    inode->blkcnt         = 0;
    inode->references     = 1;
    inode->changed        = 0;
    inode->pipe_dir       = 1;   // Must start with write.
    inode->pipe_off_read  = 0;
    inode->pipe_off_write = 0;
    inode->pipe_ref_read  = 0;
    inode->pipe_ref_write = 0;
    //
    // Add all access permissions.
    //
    inode->mode |= (S_IRWXU|S_IRWXG|S_IRWXO);
    //
    // Return the inode pointer.
    //
    return (inode);
}
