#include <kernel/fs.h>
#include <errno.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------------------
int
inode_put (inode_t *inode)
{
    int        status;
    //
    // Check for valid argument.
    //
    if (inode == NULL)
      {
        errset (EINVAL);                // Invalid argument.
        return (-1);
      }
    //
    // Check for valid references.
    //
    if (inode->references <= 0)
      {
        errset (EUNKNOWN);              // Cannot put an inode with
        return (-1);                    // zero or negative references.
      }
    //
    // Debug.
    //
    if (inode->ino != 0 && inode->sb->device == 0)
      {
        k_printf ("kernel alert: trying to close inode with device "
                  "zero, but a number different than zero!\n");
        errset (EUNKNOWN);              // Cannot put an inode with
        return (-1);                    // zero or negative references.
      }
    //
    // There is at least one reference: now the references value is
    // reduced.
    //
    inode->references--;
    inode->changed = 1;
    //
    // If `inode->ino' is zero, it means that the inode was created in
    // memory, but there is no file system for it. For example, it might
    // be a standard I/O inode create automatically for a process.
    // Inodes with number zero cannot be removed from a file system.
    //
    if (inode->ino == 0)
      {
        //
        // Nothing to do: just return.
        //
        return (0);
      }
    //
    //  References counter might be zero.
    //
    if (inode->references == 0)
      {
        //
        // Check if the inode is to be deleted (until there are
        // run time references, the inode cannot be removed).
        //
        if (inode->links == 0
         || (S_ISDIR (inode->mode) && inode->links == 1))
          {
            //
            // The inode has no more run time references and file system
            // links are also zero (or one for a directory): remove it!
            //
            status = inode_truncate (inode);
            if (status != 0)
              {
                k_perror (NULL);
              }
            //
            inode_free (inode);
            return (0);
          }
      }
    //
    // Save inode to disk and return.
    //
    return (inode_save (inode));
}
