#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
//----------------------------------------------------------------------
int
path_fix (char *path)
{
    char    new_path[PATH_MAX];
    char   *token[PATH_MAX/4];
    int     t;                          // Token index.
    int     token_size;                 // Token array effective size.
    int     comp;                       // String compare return value.
    size_t  path_size;                  // Path string size.
    //
    // Initialize token search.
    //
    token[0] = strtok (path, "/");
    //
    // Scan tokens.
    //
    for (t = 0;
         t < PATH_MAX/4 && token[t] != NULL;
         t++, token[t] = strtok (NULL, "/"))
      {
        //
        // If current token is `.', just ignore it.
        //
        comp = strcmp (token[t], ".");
        if (comp == 0)
          {
            t--;
          }
        //
        // If current token is `..', remove previous token,
        // if there is one.
        //
        comp = strcmp (token[t], "..");
        if (comp == 0)
          {
            if (t > 0)
              {
                t -= 2;
              }
            else
              {
                t = -1;
              }
          }
        //
        // `t' will be incremented and another token will be
        // found.
        //
      }
    //
    // Save the token array effective size.
    //
    token_size = t;
    //
    // Initialize the new path string.
    //
    new_path[0] = '\0';
    //
    // Build the new path string.
    //
    if (token_size > 0)
      {
        for (t = 0; t < token_size; t++)
          {
            path_size = strlen (new_path);
            strncat (new_path, "/", 2);
            strncat (new_path, token[t], PATH_MAX - path_size - 1);
          }
      }
    else
      {
        strncat (new_path, "/", 2);
      }
    //
    // Copy the new path into the original string.
    //
    strncpy (path, new_path, PATH_MAX);
    //
    // Return.
    //
    return (0);
}
