#include <kernel/fs.h>
#include <errno.h>
#include <kernel/dev.h>
//----------------------------------------------------------------------
int
sb_save (sb_t *sb)
{
    ssize_t size_written;
    addr_t  start;
    size_t  size_map;
    //
    // Check for valid argument.
    //
    if (sb == NULL)
      {
        errset (EINVAL);                        // Invalid argument.
        return (-1);
      }
    //
    // Check if the super block changed for some reason (only the
    // inode and the zone maps can change really).
    //
    if (!sb->changed)
      {
        //
        // Nothing to save.
        //
        return (0);
      }
    //
    // Something inside the super block changed: start the procedure to
    // save the inode map (recall that the super block header is not
    // saved, because it never changes).
    //
    start  = 1024;                              // After boot block.
    start += 1024;                              // After super block.
    size_map = sb->map_inode_blocks * 1024;
    size_written = dev_io ((pid_t) -1, sb->device, DEV_WRITE, start,
                sb->map_inode, size_map, NULL);
    if (size_written != size_map)
      {
        //
        // Error writing the map.
        //
        errset (EIO);                   // I/O error.
        return (-1);
      }
    //
    // Start the procedure to save the zone map.
    //
    start  = 1024;                              // After boot block.
    start += 1024;                              // After super block.
    start += (sb->map_inode_blocks * 1024);     // After inode bit map.
    size_map = sb->map_zone_blocks * 1024;
    size_written = dev_io ((pid_t) -1, sb->device, DEV_WRITE, start,
                sb->map_zone, size_map, NULL);
    if (size_written != size_map)
      {
        //
        // Error writing the map.
        //
        errset (EIO);                   // I/O error.
        return (-1);
      }
    //
    // Super block saved.
    //
    sb->changed = 0;
    //
    return (0);
}
