#include <kernel/fs.h>
#include <kernel/dev.h>
#include <errno.h>
//----------------------------------------------------------------------
int
zone_write (sb_t *sb, zno_t zone, void *buffer)
{
    size_t   size_zone;
    off_t    off_start;
    ssize_t  size_written;
    //
    // Verify if write is allowed.
    //
    if (sb->options & MOUNT_RO)
      {
        errset (EROFS);         // Read-only file system.
        return (-1);
      }
    //
    // Verify if the requested zone is inside the file system area.
    //
    if (zone >= sb->zones)
      {
        errset (EINVAL);        // Invalid argument.
        return (-1);
      }
    //
    // Write is allowed: calculate start position.
    //
    size_zone  = 1024 << sb->log2_size_zone;
    off_start  = zone;
    off_start *= size_zone;
    //
    // Write the buffer to the device.
    //
    size_written = dev_io ((pid_t) -1, sb->device, DEV_WRITE, off_start,
                           buffer, size_zone, NULL);
    if (size_written != size_zone)
      {
        errset (EIO);           // I/O error.
        return (-1);
      }
    else
      {
        return (0);
      }
}
