#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------------------
int
s_dup2 (pid_t pid, int fdn_old, int fdn_new)
{
  proc_t         *ps;
  int             status;
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Verify if `fdn_old' is a valid value.
  //
  if (fdn_old < 0 ||
      fdn_old >= OPEN_MAX || ps->fd[fdn_old].file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return (-1);
    }
  //
  // Check if `fd_old' and `fd_new' are the same.
  //
  if (fdn_old == fdn_new)
    {
      return (fdn_new);
    }
  //
  // Close `fdn_new' if it is open and copy `fdn_old' into it.
  //
  if (ps->fd[fdn_new].file != NULL)
    {
      status = s_close (pid, fdn_new);
      if (status != 0)
	{
	  return (-1);
	}
    }
  ps->fd[fdn_new].fl_flags = ps->fd[fdn_old].fl_flags;
  ps->fd[fdn_new].fd_flags = ps->fd[fdn_old].fd_flags & ~FD_CLOEXEC;
  ps->fd[fdn_new].file = ps->fd[fdn_old].file;
  ps->fd[fdn_new].file->references++;
  return (fdn_new);
}
