#ifndef _KERNEL_NET_TCP_H
#define _KERNEL_NET_TCP_H    1
//----------------------------------------------------------------------
#include <netinet/tcp.h>
#include <kernel/net.h>
//----------------------------------------------------------------------
#define TCP_HEADER_SIZE      20
#define TCP_MAX_PACKET_SIZE  NET_IP_MAX_DATA_SIZE
#define TCP_MAX_DATA_SIZE    TCP_MAX_PACKET_SIZE-TCP_HEADER_SIZE
//
#define TCP_MAX_DELAY        (CLOCKS_PER_SEC*2)
//----------------------------------------------------------------------
//
// TCP packet, for transmission.
//
typedef struct {
  struct tcphdr header;
  uint8_t       data[TCP_MAX_DATA_SIZE];
} __attribute__ ((packed)) tcp_packet_t;
//
// TCP pseudo header for checksum calculation.
//
typedef struct {
  in_addr_t saddr;
  in_addr_t daddr;
  uint8_t   zero;
  uint8_t   protocol;
  uint16_t  length;
} __attribute__ ((packed)) tcp_pseudo_header_t;
//----------------------------------------------------------------------
#define TCP_FLAG_NULL  0
#define TCP_FLAG_ACK   1
#define TCP_FLAG_PSH   2
#define TCP_FLAG_RST   4
#define TCP_FLAG_SYN   8
#define TCP_FLAG_FIN  16
//----------------------------------------------------------------------
#define TCP_TRY_READ   1        // 2^0    Wake up reading TCP process.
#define TCP_TRY_WRITE  2        // 2^1    Wake up writing TCP process.
//----------------------------------------------------------------------
int tcp_tx_raw (h_port_t sport, h_port_t dport,
                uint32_t seq, uint32_t ack_seq,
                int flags,
                h_addr_t saddr, h_addr_t daddr,
                const void *buffer, size_t size);

int tcp_tx_sock (void *sock_item);
int tcp_tx_ack  (void *sock_item);
int tcp_rx_ack  (void *sock_item, void *packet);

int tcp (void);

int tcp_connect  (void *sock_item);

int tcp_tx_rst   (void *ip_packet);

void tcp_test    (void);

void tcp_show    (h_addr_t src, h_addr_t dst, const struct tcphdr *tcphdr);

int  tcp_close   (void *sock_item);

int  tcp_rx_data (void *sock_item, void *packet);

int  tcp_status  (void *ip_packet);

//size_t tcp_rx_data_size (void *sock_item, void *packet);

//----------------------------------------------------------------------
#endif
