#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <kernel/fs.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
tcp_rx_ack (void *sock_item, void *packet)
{
  sock_t        *sock   = sock_item;
  struct iphdr  *iphdr  = packet;
  struct tcphdr *tcphdr = (struct tcphdr *)
                           &((uint8_t *) packet)[iphdr->ihl * 4];
  int            i;
  //
  // Is the ACK sequence right?
  //
  if (sock->tcp.lsq_ack != ntohl (tcphdr->ack_seq))
    {
      //
      // If it is a previous sequence, just ignore
      // the packet.
      //
      for (i = 0; i < 16; i++)
        {
          if (sock->tcp.lsq[i] == ntohl (tcphdr->ack_seq))
            {
              break;
            }
        }
      if (i >= 16)
        {
          if (DEBUG)
            {
              k_printf ("ERR loc seq: ");
              tcp_show (ntohl (iphdr->saddr), ntohl (iphdr->daddr),
                        tcphdr);

              int j;
              for (j = 0; j < 16; j++)
                {
                  if (sock->tcp.lsq[j] != 0)
                    {
                      k_printf ("%3u ", sock->tcp.lsq[j]);
                    }
                }
              k_printf ("lsq_ack=%3u ", sock->tcp.lsq_ack);
              k_printf ("\n");
            }
          //
          // The ACK is out of sequence: sorry.
          //
          tcp_tx_rst (iphdr);
        }
      return (-1);
    }
  //
  // Is the TCP sequence what we expected? But notice that, if our
  // remote expected sequence is zero, this is the first time that
  // get it, so it is right.
  //
  if (sock->tcp.rsq[sock->tcp.rsqi] != 0
      && sock->tcp.rsq[sock->tcp.rsqi] != ntohl (tcphdr->seq))
    {
      //
      // If it is a previous sequence, just ignore
      // the packet.
      //
      for (i = 0; i < 16; i++)
        {
          if (sock->tcp.rsq[i] == ntohl (tcphdr->seq))
            {
              break;
            }
        }
      if (i >= 16)
        {
          //
          // The packet is out of sequence: sorry.
          //
          if (DEBUG)
            {
              k_printf ("ERR rem seq: ");
              tcp_show (ntohl (iphdr->saddr), ntohl (iphdr->daddr),
                        tcphdr);

              int j;
              for (j = 0; j < 16; j++)
                {
                  if (sock->tcp.rsq[j] != 0)
                    {
                      k_printf ("%3u ", sock->tcp.rsq[j]);
                    }
                }
              k_printf ("\n");
            }
          tcp_tx_rst (iphdr);
        }
      return (-1);
    }
  return (0);
}
