#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <kernel/fs.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
tcp_rx_data (void *sock_item, void *packet)
{
  sock_t        *sock   = sock_item;
  struct iphdr  *iphdr  = packet;
  struct tcphdr *tcphdr = (struct tcphdr *)
                           &((uint8_t *) packet)[iphdr->ihl * 4];
  uint8_t       *recv_data;
  size_t         recv_size;
  //
  recv_data = &((uint8_t *) tcphdr)[tcphdr->doff * 4];
  recv_size = ntohs (iphdr->tot_len) - (iphdr->ihl * 4)
                                     - (tcphdr->doff * 4);
  //
  if (DEBUG)
    {
      if (recv_size > 0 && !sock->tcp.can_recv)
        {
          k_printf ("[%s] not ready to get data\n", __func__);
        }
    }
  //
  // If we receive zero data, it is ok.
  //
  if (recv_size == 0)
    {
      //
      // Nothing to do, but there is no error.
      //
      return (0);
    }
  if (recv_size < 0)
    {
      return (-1);
    }
  if (recv_size > 0 && !sock->tcp.can_recv)
    {
      return (-1);
    }
  //
  // Check the size.
  //
  if (recv_size > sizeof (sock->tcp.recv_data))
    {
      k_printf ("[%s] cannot accept a packet "
                "payload of %u bytes; packet "
                "truncated at %u bytes!\n",
                __func__, recv_size,
                sizeof (sock->tcp.recv_data));
      //
      recv_size = sizeof (sock->tcp.recv_data);
    }
  //
  memcpy (sock->tcp.recv_data, recv_data, recv_size);
  sock->tcp.recv_size  = recv_size;
  sock->tcp.recv_index = sock->tcp.recv_data;
  //
  // Must ACK back for the data received.
  //
  // The remote sequence that we will expect next time
  // and the local sequence, expected from the other
  // side.
  //
  sock->tcp.rsq[++sock->tcp.rsqi] = ntohl (tcphdr->seq) + recv_size;
  sock->tcp.lsq[++sock->tcp.lsqi] = sock->tcp.lsq_ack;
  //
  tcp_tx_ack (sock);
  //
  // Now the received data, if any, is to be read.
  //
  sock->tcp.can_recv = 0;
  sock->tcp.can_read = 1;
  //
  //
  //
  return (0);
}
