#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <kernel/fs.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
tcp_tx_ack (void *sock_item)
{
  sock_t             *sock = sock_item;
  tcp_packet_t        packet;
  tcp_pseudo_header_t pseudo;
  uint16_t            checksum;
  //
  if (sock == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  if (sock->laddr == 0 || sock->raddr == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  if (sock->lport == 0 || sock->rport == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Prepare the TCP packet.
  //
  memset (&packet.header, 0, sizeof (struct tcphdr));
  //
  packet.header.source   = htons (sock->lport);
  packet.header.dest     = htons (sock->rport);
  packet.header.seq      = htonl (sock->tcp.lsq[sock->tcp.lsqi]);
  packet.header.ack_seq  = htonl (sock->tcp.rsq[sock->tcp.rsqi]);
  packet.header.doff     = (sizeof (struct tcphdr) / 4);        // 5
  packet.header.ack      = 1;
  packet.header.window   = htons (TCP_MSS);        // Minimal window
  packet.header.check    = 0;
  //
  // Prepare the pseudo header.
  //
  pseudo.saddr    = htonl (sock->laddr);
  pseudo.daddr    = htonl (sock->raddr);
  pseudo.zero     = 0;
  pseudo.protocol = IPPROTO_TCP;
  pseudo.length   = htons (sizeof (struct tcphdr));
  //
  // Now set the header checksum.
  //
  checksum = ~(ip_checksum ((void *) &packet, sizeof (struct tcphdr),
                            (void *) &pseudo,
                            sizeof (tcp_pseudo_header_t)));
  if (checksum == 0)
    {
      checksum = 0xFFFF;
    }
  packet.header.check = htons (checksum);
  //
  // Send to the lower network level.
  //
  if (DEBUG)
    {
      tcp_show (sock->laddr, sock->raddr, (struct tcphdr *) &packet);
    }
  return (ip_tx (sock->laddr, sock->raddr, (int) IPPROTO_TCP,
                 &packet, sizeof (struct tcphdr)));
}
