#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
void
proc_sig_core (pid_t pid, int sig)
{
    addr_t address_text;
    addr_t address_data;
    size_t domain_text;
    size_t domain_data;
    size_t extra_data;
    //
    if (proc_sig_status (pid, sig))
      {
        if (proc_sig_ignore (pid, sig))
          {
            proc_sig_off (pid, sig);
          }
        else
          {
            if (proc_table[pid].sig_handler[sig] != (uintptr_t) NULL)
              {
                proc_sig_handler (pid, sig);
              }
            else
              {
                //
                // Save process addresses and sizes (might be useful if
                // we want to try to exit the process before core dump.
                //
                address_text = proc_table[pid].address_text;
                address_data = proc_table[pid].address_data;
                domain_text  = proc_table[pid].domain_text;
                domain_data  = proc_table[pid].domain_data;
                extra_data   = proc_table[pid].extra_data;
                //
                // Core dump: the process who formally writes the file
                // is the terminating one.
                //
                if (domain_data == 0)
                  {
                    proc_dump_memory (pid, address_text,
                                      domain_text + extra_data, "core");
                  }
                else
                  {
                    proc_dump_memory (pid, address_text,
                                      domain_text, "core.text");
                    proc_dump_memory (pid, address_data,
                                      domain_data + extra_data,
                                      "core.data");
                  }
                //
                // The signal, translated to negative, is returned (but
                // the effective value received by the application will
                // be cutted, leaving only the low 8 bit).
                //
                s__exit (pid, -sig);
              }
          }
      }
}
