#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stddef.h>
//----------------------------------------------------------------------
struct dirent *
readdir (DIR *dp)
{
    ssize_t size;
    //
    // Check for a valid argument.
    //
    if (dp == NULL)
      {
        //
        // Not a valid pointer.
        //
        errset (EBADF);         // Invalid directory.
        return (NULL);
      }
    //
    // Check if it is an open directory stream.
    //
    if (dp->fdn < 0)
      {
        //
        // The stream is closed.
        //
        errset (EBADF);         // Invalid directory.
        return (NULL);
      }
    //
    // Read the directory.
    //
    size = read (dp->fdn, &(dp->dir),
                 (size_t) 16);
    //
    // Fix the null termination, if the name is very long.
    //
    dp->dir.d_name[NAME_MAX] = '\0';
    //
    // Check what was read.
    //
    if (size == 0)
      {
        //
        // End of directory, but it is not an error.
        //
        return (NULL);
      }
    //
    if (size < 0)
      {
        //
        // This is an error. The variable `errno' is already set.
        //
        errset (errno);
        return (NULL);
      }
    //
    if (dp->dir.d_ino == 0)
      {
        //
        // This is a null directory record.
        // Should try to read the next one.
        //
        return (readdir (dp));
      }
    //
    if (strlen (dp->dir.d_name) == 0)
      {
        //
        // This is a bad directory record: try to read next.
        //
        return (readdir (dp));
      }
    //
    // A valid directory record should be available now.
    //
    return (&(dp->dir));
}
