#ifndef _FCNTL_H
#define _FCNTL_H        1

#include <sys/types.h>  // mode_t
                        // off_t
                        // pid_t
//----------------------------------------------------------------------
// Values for the second parameter of function `fcntl()'.
//----------------------------------------------------------------------
#define F_DUPFD         0       // Duplicate file descriptor.
#define F_GETFD         1       // Get file descriptor flags.
#define F_SETFD         2       // Set file descriptor flags.
#define F_GETFL         3       // Get file status flags.
#define F_SETFL         4       // Set file status flags.
#define F_GETLK         5       // Get record locking information.
#define F_SETLK         6       // Set record locking information.
#define F_SETLKW        7       // Set record locking information;
                                // wait if blocked.
#define F_GETOWN        8       // Set owner of socket.
#define F_SETOWN        9       // Get owner of socket.
//----------------------------------------------------------------------
// Flags to be set with:
//   fcntl (fd, F_SETFD, ...);
//----------------------------------------------------------------------
#define FD_CLOEXEC      1       // Close the file descriptor upon
                                // execution of an exec() family
                                // function.
//----------------------------------------------------------------------
// Values for type `l_type', used for record locking with `fcntl()'.
//----------------------------------------------------------------------
#define F_RDLCK         0       // Read lock.
#define F_WRLCK         1       // Write lock.
#define F_UNLCK         2       // Remove lock.
//----------------------------------------------------------------------
// Flags for file creation, in place of `oflag' parameter for function
// `open()'.
//----------------------------------------------------------------------
#define O_CREAT         000010  // Create file if it does not exist.
#define O_EXCL          000020  // Exclusive use flag.
#define O_NOCTTY        000040  // Do not assign a controlling terminal.
#define O_TRUNC         000100  // Truncation flag.
//----------------------------------------------------------------------
// Flags for the file status, used with `open()' and `fcntl()'.
//----------------------------------------------------------------------
#define O_APPEND        000200  // Write append.
#define O_DSYNC         000400  // Synchronized write operations.
#define O_NONBLOCK      001000  // Non-blocking mode.
#define O_RSYNC         002000  // Synchronized read operations.
#define O_SYNC          004000  // Synchronized read and write.
//----------------------------------------------------------------------
// File access mask selection.
//----------------------------------------------------------------------
#define O_ACCMODE       000003  // Mask to select the last three bits,
                                // used to specify the main access
                                // modes: read, write and both.
//----------------------------------------------------------------------
// Main access modes.
//----------------------------------------------------------------------
#define O_RDONLY        000001                  // Read.
#define O_WRONLY        000002                  // Write.
#define O_RDWR          (O_RDONLY | O_WRONLY)   // Both read and write.
//----------------------------------------------------------------------
// Structure `flock', used to file lock for POSIX standard. It is not
// used inside os32.
//----------------------------------------------------------------------
struct flock {
    short int l_type;   // Type of lock: F_RDLCK, F_WRLCK, or F_UNLCK.
    short int l_whence; // Start reference point.
    off_t     l_start;  // Offset, from `l_whence', for the area start.
    off_t     l_len;    // Locked area size. Zero means up to the end of
                        // the file.
    pid_t     l_pid;    // The process id blocking the area.
};
//----------------------------------------------------------------------
// Function prototypes.
//----------------------------------------------------------------------
int creat (const char *path, mode_t mode);
int fcntl (int fdn, int cmd, ...);
int open  (const char *path, int oflags, ...);
//----------------------------------------------------------------------

#endif
