#ifndef _STDIO_H
#define _STDIO_H        1
//----------------------------------------------------------------------
#include <restrict.h>
#include <stdarg.h>
#include <stdint.h>
#include <limits.h>
#include <NULL.h>
#include <size_t.h>
#include <sys/types.h>
#include <SEEK.h>       // SEEK_CUR, SEEK_SET, SEEK_END
//----------------------------------------------------------------------
#define BUFSIZ              8192  // At least the file system max zone
                                  // size.
#define _IOFBF                 0  // Input-output fully buffered.
#define _IOLBF                 1  // Input-output line buffered.
#define _IONBF                 2  // Input-output with no buffering.

#define L_tmpnam    FILENAME_MAX  // <limits.h>

#define FOPEN_MAX       OPEN_MAX  // <limits.h>
#define FILENAME_MAX    NAME_MAX  // <limits.h>
#define TMP_MAX           0x7FFF

#define EOF                 (-1)  // Must be a negative value.
//----------------------------------------------------------------------
typedef off_t     fpos_t;   // `off_t' defined in <sys/types.h>.

typedef struct {
    int           fdn;      // File descriptor number.
    char          error;    // Error indicator.
    char          eof;      // End of file indicator.
} FILE;

extern FILE _stream[];      // Defined inside `lib/stdio/FILE.c'.

#define stdin   (&_stream[0])
#define stdout  (&_stream[1])
#define stderr  (&_stream[2])
//----------------------------------------------------------------------
void     clearerr   (FILE *fp);
int      fclose     (FILE *fp);
int      feof       (FILE *fp);
int      ferror     (FILE *fp);
int      fflush     (FILE *fp);
int      fgetc      (FILE *fp);
int      fgetpos    (FILE *restrict fp, fpos_t *restrict pos);
char    *fgets      (char *restrict string, int n, FILE *restrict fp);
int      fileno     (FILE *fp);
FILE    *fopen      (const char *path, const char *mode);
int      fprintf    (FILE *fp, char *restrict format, ...);
int      fputc      (int c, FILE *fp);
int      fputs      (const char *restrict string, FILE *restrict fp);
size_t   fread      (void *restrict buffer, size_t size, size_t nmemb,
                     FILE *restrict fp);
FILE    *freopen    (const char *restrict path,
                     const char *restrict mode,
                     FILE *restrict fp);
int      fscanf     (FILE *restrict fp, const char *restrict format,
                     ...);
int      fseek      (FILE *fp, long int offset, int whence);
int      fsetpos    (FILE *fp, fpos_t *pos);
long int ftell      (FILE *fp);
off_t    ftello     (FILE *fp);
size_t   fwrite     (const void *restrict buffer, size_t size,
                     size_t nmemb, FILE *restrict fp);
#define  getc(p)    (fgetc (p))
int      getchar    (void);
char    *gets       (char *string);
void     perror     (const char *string);
int      printf     (const char *restrict format, ...);
#define  putc(c, p) (fputc ((c), (p)))
int      putchar    (int c);
int      puts       (const char *string);
void     rewind     (FILE *fp);
int      scanf      (const char *restrict format, ...);
void     setbuf     (FILE *restrict fp, char *restrict buffer);
int      setvbuf    (FILE *restrict fp, char *restrict buffer,
                     int buf_mode, size_t size);
int      snprintf   (char *restrict string, size_t size,
                     const char *restrict format, ...);
int      sprintf    (char *restrict string, const char *restrict format,
                     ...);
int      sscanf     (char *restrict string, const char *restrict format,
                     ...);
int      vfprintf   (FILE *fp, char *restrict format, va_list arg);
int      vfscanf    (FILE *restrict fp, const char *restrict format,
                     va_list arg);
int      vprintf    (const char *restrict format, va_list arg);
int      vscanf     (const char *restrict format, va_list ap);
int      vsnprintf  (char *restrict string, size_t size,
                     const char *restrict format, va_list arg);
int      vsprintf   (char *restrict string, const char *restrict format,
                     va_list arg);
int      vsscanf    (const char *string, const char *format,
                     va_list ap);
//----------------------------------------------------------------------
#endif
