#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
//----------------------------------------------------------------------
uintptr_t _alloc_start = 0;
//----------------------------------------------------------------------
static int   _alloc_init (void);
static void *_malloc     (size_t size);
//----------------------------------------------------------------------
void *
malloc (size_t size)
{
  void *pstatus;
  int   status;
  //
  // Verify to have initialized the allocation memory.
  //
  if (_alloc_start == 0)
    {
      status = _alloc_init ();
      if (status < 0)
        {
          errset (ENOMEM);
          return (NULL);
        }
    }
  //
  // Try to allocate as usual.
  //
  pstatus = _malloc (size);
  //
  if (pstatus == NULL)
    {
      //
      // Try to increase memory for the process.
      //
      pstatus = sbrk (size);
      if (pstatus == NULL)
        {
          //
          // Sorry: no way to get memory.
          //
          errset (ENOMEM);
          return (NULL);
        }
      //
      // Ok. Now try again to allocate memory.
      //
      return (_malloc (size));
    }
  else
    {
      //
      // The first allocation was successful.
      //
      return (pstatus);
    }
}
//----------------------------------------------------------------------
static int
_alloc_init (void)
{
  uintptr_t      start;
  uintptr_t      end;
  _alloc_head_t *head;
  size_t         available;
  //
  // Get size.
  //
  if (_alloc_start == 0)
    {
      _alloc_start = (uintptr_t) sbrk (0);
    }
  //
  start     = _alloc_start;
  end       = (uintptr_t) sbrk (0);
  available = end - start;
  //
  // Check available space.
  //
  if (available < ((sizeof (_alloc_head_t)) * 2))
    {
      //
      // Try to get a little memory.
      //
      sbrk ((sizeof (_alloc_head_t)) * 2);
      end = (uintptr_t) sbrk (0);
      available = end - start;
      if (available < ((sizeof (_alloc_head_t)) * 2))
        {
          //
          // Sorry!
          //
          return (-1);
        }
    }
  //
  // Prepare the list main node.
  //
  head = (_alloc_head_t *) start;
  //
  // Init the first free block, that points to itself, as it is
  // the only one.
  //
  head->allocated = 0;
  head->next      = (start / (sizeof (_alloc_head_t)));
  //
  // Ok.
  //
  return (0);
}
//----------------------------------------------------------------------
static void *
_malloc (size_t size)
{
  uintptr_t  start = _alloc_start;
  uintptr_t  end   = (uintptr_t) sbrk (0);
  _alloc_head_t *head  = (void *) start;
  size_t     actual_size;
  uintptr_t  current;
  uintptr_t  next;
  uintptr_t  new;
  uintptr_t  up_to;
  int        counter;
  //
  // Arrotonda in eccesso il valore di «size», in modo che sia un
  // multiplo della dimensione di «_alloc_head_t». Altrimenti, la
  // collocazione dei blocchi successivi può avvenire in modo
  // non allineato.
  //
  size = (size + (sizeof (_alloc_head_t)) -1);
  size = size / (sizeof (_alloc_head_t));
  size = size * (sizeof (_alloc_head_t));
  //
  // Cerca un blocco libero di dimensione sufficiente.
  //
  counter = 2;
  while (counter)
    {
      //
      // Annota la posizione attuale e quella successiva.
      //
      current = (uintptr_t) head;
      next    = head->next * (sizeof (_alloc_head_t));
      //
      if (next == start)
        {
          up_to = end;
        }
      else
        {
          up_to = next;
        }
      //
      // Se è stato raggiunto il primo elemento, decrementa il
      // contatore di una unità. Se è già a zero, esce.
      //
      if (current == start)
        {
          counter--;
          if (counter == 0) break;
        }
      //
      // Controlla se si tratta di un blocco libero.
      //

      if (! head->allocated)
        {
          //
          // Il blocco è libero: si deve determinarne la dimensione.
          //
          if (current == start && next == start)
            {
              //
              // Si tratta del primo e unico elemento della lista.
              //
              actual_size = end - start - (sizeof (_alloc_head_t));
            }
          else
            {
              actual_size = up_to - current - (sizeof (_alloc_head_t));
            }
          //
          // Si verifica che sia capiente.
          //
          if (actual_size >= size + ((sizeof (_alloc_head_t)) * 2))
            {
              //
              // C'è spazio per dividere il blocco.
              //
              new = current + size + (sizeof (_alloc_head_t));
              //
              // Aggiorna l'intestazione attuale.
              //
              head->allocated = 1;
              head->next = new / (sizeof (_alloc_head_t));
              //
              // Predispone l'intestazione successiva.
              //
              head = (void *) new;
              head->allocated = 0;
              head->next = next / (sizeof (_alloc_head_t));
              //
              // Restituisce l'indirizzo iniziale dello spazio libero,
              // successivo all'intestazione.
              //
              return (void *) (current + (sizeof (_alloc_head_t)));
            }
          else if (actual_size >= size)
            {
              //
              // Il blocco va usato per intero.
              //
              head->allocated = 1;
              //
              // Restituisce l'indirizzo iniziale dello spazio libero,
              // successivo all'intestazione.
              //
              return (void *) (current + (sizeof (_alloc_head_t)));
            }
        }
      //
      // Il blocco è allocato, oppure è di dimensione insufficiente;
      // pertanto occorre passare alla posizione successiva.
      //
      head = (void *) next;
    }
  //
  // Essendo terminato il ciclo precedente, vuol dire
  // che non ci sono spazi disponibili.
  //
  errset (ENOMEM);
  return NULL;
}
